require File.expand_path('../../../application', __FILE__)

config = Inf.server.unicorn

# workers
worker_processes config.workers
 
# worker response timeout
timeout config.timeout
 
# iisten on interface
listen "#{config.interface}:#{config.port}", backlog: config.backlog
# listen on socket
listen config.socket_file, backlog: config.backlog
# pid file
pid config.pid_file

# logs
stderr_path = config.log_file
stdout_path = config.log_file
logger = Logger.new config.log_file
logger logger

# general
check_client_connection false
preload_app true

if GC.respond_to?(:copy_on_write_friendly=)
  GC.copy_on_write_friendly = true
end

before_fork do |server, worker|
  ActiveRecord::Base.connection.disconnect!
  old_pid_file = config.pid_file + '.oldbin'
  if File.exists? old_pid_file
    begin
      Process.kill 'QUIT', File.read(old_pid_file).strip.to_i
    rescue Errno::ENOENT, Errno::ESRCH
    end
  end
end
 
after_fork do |server, worker|
  ActiveRecord::Base.establish_connection
  begin
    uid, gid = Process.euid, Process.egid
    user config.user, config.group
    target_uid = Etc.getpwnam(config.user).uid
    target_gid = Etc.getgrnam(config.group).gid
    worker.tmp.chown target_uid, target_gid
    if uid != target_uid || gid != target_gid
      Process.initgroups config.user, target_gid
      Process::GID.change_privilege target_gid
      Process::UID.change_privilege target_uid
    end
  rescue ArgumentError => error
    if error.message.include? "can't find user"
      unless Rails.env.production?
        logger.info { "Could not change to user/group #{config.user}:#{config.group} but ignoring because environment is not production" }
      else
        logger.fatal { "Could not change to user/group #{config.user}:#{config.group}" }
        Process.kill 'QUIT', server.pid.to_i
      end
    else
      logger.fatal { "#{error.class}: #{error.message}\n" + error.backtrace.join("\n") }
      Process.kill 'QUIT', server.pid.to_i
    end
  end
end
