require 'test_helper'

class Infrastructure::DatabasesTest < MiniTest::Test

  def setup

    @orm = Class.new do
      class << self
        def establish_connection(config); config end
        def connection
          Class.new do
            def tables; ['test'] end
            def drop_database(database); database end
            def create_database(database); database end
          end.new
        end
      end
    end
    @instance = Infrastructure::Databases.new @orm, 'env1'
    @instance.configure do
      config.system = { database: 'system_database' }
      config.defaults = { global_default: 'global_default' }
      config.env1 = {
        defaults: { environment_default: 'environment_default', id: 'id1' },
        id1: {},
        id2: { database: 'override', environment_default: 'override', global_default: 'override' }
      }
      config.env2 = {
        defaults: { environment_default: 'environment_default', global_default: 'environment_override' },
        id1: {},
        id2: { database: 'override', environment_default: 'override', global_default: 'override' }
      }
    end
  end

  def test_accessors
    assert_equal :env1, @instance.environment, 'environment accessor'
    assert @instance.orm == @orm, 'orm accessor'
  end

  def test_merge_defaults
    assert_equal 'global_default', @instance.env1.id1.global_default
    assert_equal 'environment_override', @instance.env2.id1.global_default
    assert_equal 'override', @instance.env1.id2.global_default
    assert_equal 'override', @instance.env2.id2.global_default
    assert_equal 'environment_default', @instance.env1.id1.environment_default
    assert_equal 'override', @instance.env1.id2.environment_default
    assert_equal 'environment_default', @instance.env1.id1.environment_default
    assert_equal 'override', @instance.env1.id2.environment_default
    assert_equal 'id1', @instance.env1.id1.id
    assert_equal 'id2', @instance.env1.id2.id
    assert_equal 'id1', @instance.env2.id1.id
    assert_equal 'id2', @instance.env2.id2.id
    assert_equal 'env1_id1', @instance.env1.id1.database
    assert_equal 'override', @instance.env1.id2.database
    assert_equal 'env2_id1', @instance.env2.id1.database
    assert_equal 'override', @instance.env2.id2.database
  end

  def test_connection
    refute @instance.connection.is_a?(Class)
  end

  def test_tables
    assert ['test'] == @instance.tables
  end

  def test_environment_configs
    assert_equal @instance.env1, @instance.environment_configs
    assert_equal @instance.env2, @instance.environment_configs(:env2)
  end

  def test_environment_config
    assert_equal @instance.env1.id1.to_h, @instance.environment_config(:id1)
    assert_equal @instance.env1.id2.to_h, @instance.environment_config(:id2)
    assert Hash[a: :b] == @instance.environment_config(a: :b)
    assert_raises ArgumentError do
      @instance.environment_config(:unknown)
    end
    assert_raises ArgumentError do
      @instance.environment_defaults[:id] = 'id3'
      @instance.environment_config(:id3)
    end
  end

  def test_drop
    assert_equal @instance.env1.id1.database, @instance.drop(:id1)
    assert_equal 'test', @instance.drop(database: 'test')
  end

  def test_create
    assert_equal @instance.env1.id1.database, @instance.create(:id1)
    assert_equal 'test', @instance.create(database: 'test')
  end

  def test_connect
    assert_equal @instance.env1.id1.to_h, @instance.connect(:id1)
  end

  def test_connect!
    assert_equal @instance.env1.id1.to_h, @instance.connect(:id1)
  end

  def test_databases
    assert ['env1_id1', 'override'] == @instance.databases
  end

end
